@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 """Create direct, total and mediation Graphviz graph from dot_str using pydot."""
 
-from typing import Dict
+from typing import Dict, Sequence, Optional
 from itertools import chain
 import numpy as np
 from numpy import amax, array_equal, allclose, isnan, logical_and
@@ -59,27 +59,6 @@ def color_str(wei, base, line_colored, color, colortrans):
     return col_str
 
 
-def single_nodes(xnodes, ynodes, mat_id):
-    """find single nodes, without incoming or outgoing edges,
-    given either id, ed or fd identification matrix"""
-
-    mat_id_df = DataFrame(mat_id, ynodes, xnodes)
-
-    if not array_equal(xnodes, ynodes):
-        # x nodes
-        mat_id_df = mat_id_df.loc[:, (mat_id_df == 0).all(axis=0)]
-        sing_nod = list(mat_id_df)
-    else:
-        # y nodes
-        mat_id_df = mat_id_df.loc[
-            :, logical_and((mat_id_df == 0).all(axis=0), (mat_id_df == 0).all(axis=1))
-        ]
-        mat_id_df = mat_id_df.loc[(mat_id_df == 0).all(axis=1), :]
-        sing_nod = list(mat_id_df)
-
-    return sing_nod
-
-
 def dot(
     # main graph data
     xnodes,
@@ -95,15 +74,13 @@ def dot(
     """create inner graphviz dot_string,
     do not show edges with exact zero weight, irrespective of id matrix"""
 
-    # single nodes
-    sing_nod = single_nodes(xnodes, ynodes, weights)
-
+    xnodes_with_edge = set()
     dot_str = ""
     # edges
     for row, ynode in enumerate(ynodes):
         for col, xnode in enumerate(xnodes):
             wei = weights[row, col]
-            if isnan(wei) or wei == 0 or xnode == ynode:
+            if isnan(wei) or xnode == ynode:
                 continue
 
             if show_in_percent:
@@ -115,10 +92,12 @@ def dot(
             dot_str += '         "{}" -> "{}" [label = "{}"{}];\n'.format(
                 xnode, ynode, wei_str, col_str
             )
+            xnodes_with_edge.add(xnode)
+            xnodes_with_edge.add(ynode)
 
     # nodes
     for i, xnode in enumerate(xnodes):
-        if xnode in sing_nod:
+        if xnode not in xnodes_with_edge:
             continue
 
         if nodeff is not None and not isnan(nodeff[i]):
@@ -231,6 +210,7 @@ def create_graphs(
     node_name,
     show_nr_indiv,
     final_var_in_percent=False,
+    ids: Optional[Sequence[str]] = None,
 ):
     """creates direct, total and mediation graph,
     for theoretical model and estimated model"""
@@ -273,14 +253,19 @@ def create_graphs(
     mediation_indiv_graphs = []
     print()
 
-    for i in range(show_nr_indiv):
+    for i in range(min(show_nr_indiv, len(graph_json["mx_indivs"]))):
+        if ids:
+            item_id = ids[i]
+        else:
+            item_id = str(i)
+
         # compute color base for each individual separately
         # using _indiv quantities based on _theo quantities times absolute deviation from median
         print("Generate graphs for individual {:5}".format(i))
 
         # IDE
         direct_indiv_graph = make_graph(
-            "IDE" + "_" + str(i),
+            f"IDE_{item_id}",
             (graph_json["mx_indivs"][i], None),
             (graph_json["my_indivs"][i], None),
             color=True,
@@ -289,7 +274,7 @@ def create_graphs(
 
         # IME
         mediation_indiv_graph = make_graph(
-            "IME" + "_" + str(i),
+            f"IME_{item_id}",
             (graph_json["eyx_indivs"][i], np.array(graph_json["exj_indivs"])[:, i]),
             (graph_json["eyy_indivs"][i], np.array(graph_json["eyj_indivs"])[:, i]),
             color=True,
@@ -299,7 +284,7 @@ def create_graphs(
 
         # ITE
         total_indiv_graph = make_graph(
-            "ITE" + "_" + str(i),
+            f"ITE_{item_id}",
             (graph_json["ex_indivs"][i], None),
             (graph_json["ey_indivs"][i], None),
             color=True,
