@@ -23,13 +23,15 @@ def fromstring(xml_string, replacements=None, **kwargs):
 
     return drawing
 
+
 def fromfile(path, replacements=None, **kwargs):
     """create svg graphics from file"""
 
-    with open(path, 'rb') as fhandle:
+    with open(path, "rb") as fhandle:
         xml_string = fhandle.read()
     return fromstring(xml_string, replacements, **kwargs)
 
+
 def fromtemplate(path, replacements=None, **kwargs):
     """create svg graphics from template"""
 
